@@ -28,6 +28,7 @@
 # USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
 import time
+from ansible.module_utils.six import iteritems
 
 try:
     from cs import CloudStack, CloudStackException, read_config
@@ -105,12 +106,14 @@ class AnsibleCloudStack(object):
         self.account = None
         self.project = None
         self.ip_address = None
+        self.network = None
+        self.vpc = None
         self.zone = None
         self.vm = None
+        self.vm_default_nic = None
         self.os_type = None
         self.hypervisor = None
         self.capabilities = None
-        self.tags = None
 
 
     def _connect(self):
@@ -146,7 +149,7 @@ class AnsibleCloudStack(object):
 
 
     def has_changed(self, want_dict, current_dict, only_keys=None):
-        for key, value in want_dict.iteritems():
+        for key, value in want_dict.items():
 
             # Optionally limit by a list of keys
             if only_keys and key not in only_keys:
@@ -157,7 +160,17 @@ class AnsibleCloudStack(object):
                 continue
 
             if key in current_dict:
-                if isinstance(current_dict[key], (int, long, float, complex)):
+                if isinstance(value, (int, float, long, complex)):
+                    # ensure we compare the same type
+                    if isinstance(value, int):
+                        current_dict[key] = int(current_dict[key])
+                    elif isinstance(value, float):
+                        current_dict[key] = float(current_dict[key])
+                    elif isinstance(value, long):
+                        current_dict[key] = long(current_dict[key])
+                    elif isinstance(value, complex):
+                        current_dict[key] = complex(current_dict[key])
+
                     if value != current_dict[key]:
                         return True
                 else:
@@ -183,6 +196,58 @@ class AnsibleCloudStack(object):
         return my_dict
 
 
+    def get_vpc(self, key=None):
+        """Return a VPC dictionary or the value of given key of."""
+        if self.vpc:
+            return self._get_by_key(key, self.vpc)
+
+        vpc = self.module.params.get('vpc')
+        if not vpc:
+            return None
+
+        args = {
+            'account': self.get_account(key='name'),
+            'domainid': self.get_domain(key='id'),
+            'projectid': self.get_project(key='id'),
+            'zoneid': self.get_zone(key='id'),
+        }
+        vpcs = self.cs.listVPCs(**args)
+        if not vpcs:
+            self.module.fail_json(msg="No VPCs available.")
+
+        for v in vpcs['vpc']:
+            if vpc in [v['displaytext'], v['name'], v['id']]:
+                self.vpc = v
+                return self._get_by_key(key, self.vpc)
+        self.module.fail_json(msg="VPC '%s' not found" % vpc)
+
+
+    def get_network(self, key=None):
+        """Return a network dictionary or the value of given key of."""
+        if self.network:
+            return self._get_by_key(key, self.network)
+
+        network = self.module.params.get('network')
+        if not network:
+            return None
+
+        args = {
+            'account': self.get_account('name'),
+            'domainid': self.get_domain('id'),
+            'projectid': self.get_project('id'),
+            'zoneid': self.get_zone('id'),
+        }
+        networks = self.cs.listNetworks(**args)
+        if not networks:
+            self.module.fail_json(msg="No networks available.")
+
+        for n in networks['network']:
+            if network in [n['displaytext'], n['name'], n['id']]:
+                self.network = n
+                return self._get_by_key(key, self.network)
+        self.module.fail_json(msg="Network '%s' not found" % network)
+
+
     def get_project(self, key=None):
         if self.project:
             return self._get_by_key(key, self.project)
@@ -224,6 +289,32 @@ class AnsibleCloudStack(object):
         return self._get_by_key(key, self.ip_address)
 
 
+    def get_vm_guest_ip(self):
+        vm_guest_ip = self.module.params.get('vm_guest_ip')
+        default_nic = self.get_vm_default_nic()
+
+        if not vm_guest_ip:
+            return default_nic['ipaddress']
+
+        for secondary_ip in default_nic['secondaryip']:
+            if vm_guest_ip == secondary_ip['ipaddress']:
+                return vm_guest_ip
+        self.module.fail_json(msg="Secondary IP '%s' not assigned to VM" % vm_guest_ip)
+
+
+    def get_vm_default_nic(self):
+        if self.vm_default_nic:
+            return self.vm_default_nic
+
+        nics = self.cs.listNics(virtualmachineid=self.get_vm(key='id'))
+        if nics:
+            for n in nics['nic']:
+                if n['isdefault']:
+                    self.vm_default_nic = n
+                    return self.vm_default_nic
+        self.module.fail_json(msg="No default IP address of VM '%s' found" % self.module.params.get('vm'))
+
+
     def get_vm(self, key=None):
         if self.vm:
             return self._get_by_key(key, self.vm)
@@ -253,6 +344,9 @@ class AnsibleCloudStack(object):
         zone = self.module.params.get('zone')
         zones = self.cs.listZones()
 
+        if not zones:
+            self.module.fail_json(msg="No zones available. Please create a zone first")
+
         # use the first zone if no zone param given
         if not zone:
             self.zone = zones['zone'][0]
@@ -345,19 +439,9 @@ class AnsibleCloudStack(object):
 
 
     def get_tags(self, resource=None):
-        if not self.tags:
-            args = {}
-            args['projectid'] = self.get_project(key='id')
-            args['account'] = self.get_account(key='name')
-            args['domainid'] = self.get_domain(key='id')
-            args['resourceid'] = resource['id']
-            response = self.cs.listTags(**args)
-            self.tags = response.get('tag', [])
-
         existing_tags = []
-        if self.tags:
-            for tag in self.tags:
-                existing_tags.append({'key': tag['key'], 'value': tag['value']})
+        for tag in resource.get('tags',[]):
+            existing_tags.append({'key': tag['key'], 'value': tag['value']})
         return existing_tags
 
 
@@ -395,8 +479,7 @@ class AnsibleCloudStack(object):
             if tags is not None:
                 self._process_tags(resource, resource_type, self._tags_that_should_not_exist(resource, tags), operation="delete")
                 self._process_tags(resource, resource_type, self._tags_that_should_exist_or_be_updated(resource, tags))
-                self.tags = None
-                resource['tags'] = self.get_tags(resource)
+                resource['tags'] = tags
         return resource
 
 
@@ -431,12 +514,12 @@ class AnsibleCloudStack(object):
         if resource:
             returns = self.common_returns.copy()
             returns.update(self.returns)
-            for search_key, return_key in returns.iteritems():
+            for search_key, return_key in returns.items():
                 if search_key in resource:
                     self.result[return_key] = resource[search_key]
 
             # Bad bad API does not always return int when it should.
-            for search_key, return_key in self.returns_to_int.iteritems():
+            for search_key, return_key in self.returns_to_int.items():
                 if search_key in resource:
                     self.result[return_key] = int(resource[search_key])
 
