@@ -50,7 +50,7 @@ options:
     description:
       - The type of DNS record to create
     required: true
-    choices: [ 'A', 'CNAME', 'MX', 'AAAA', 'TXT', 'PTR', 'SRV', 'SPF', 'NS' ]
+    choices: [ 'A', 'CNAME', 'MX', 'AAAA', 'TXT', 'PTR', 'SRV', 'SPF', 'NS', 'SOA' ]
   alias:
     description:
       - Indicates if this is an alias record.
@@ -64,6 +64,12 @@ options:
     required: false
     version_added: "1.9"
     default: null
+  alias_evaluate_target_health:
+    description:
+      - Whether or not to evaluate an alias target health. Useful for aliases to Elastic Load Balancers.
+    required: false
+    version_added: "2.1"
+    default: false
   value:
     description:
       - The new value when creating a DNS record.  Multiple comma-spaced values are allowed for non-alias records.  When deleting a record all values for the record must be specified or Route53 will not delete it.
@@ -87,7 +93,7 @@ options:
     version_added: "1.9"
   identifier:
     description:
-      - Weighted and latency-based resource record sets only. An identifier
+      - Have to be specified for Weighted, latency-based and failover resource record sets only. An identifier
         that differentiates among multiple resource record sets that have the
         same combination of DNS name and type.
     required: false
@@ -120,7 +126,7 @@ options:
   failover:
     description:
       - Failover resource record sets only. Whether this is the primary or
-        secondary resource record set.
+        secondary resource record set. Allowed values are PRIMARY and SECONDARY
     required: false
     default: null
     version_added: "2.0"
@@ -131,14 +137,28 @@ options:
     required: false
     default: null
     version_added: "2.0"
-author: "Bruce Pennypacker (@bpennypacker)"
+  wait:
+    description:
+      - Wait until the changes have been replicated to all Amazon Route 53 DNS servers.
+    required: false
+    default: no
+    version_added: "2.1"
+  wait_timeout:
+    description:
+      - How long to wait for the changes to be replicated, in seconds.
+    required: false
+    default: 300
+    version_added: "2.1"
+author:
+  - "Bruce Pennypacker (@bpennypacker)"
+  - "Mike Buzzetti <mike.buzzetti@gmail.com>"
 extends_documentation_fragment: aws
 '''
 
 # FIXME: the command stuff should have a more state like configuration alias -- MPD
 
 EXAMPLES = '''
-# Add new.foo.com as an A record with 3 IPs
+# Add new.foo.com as an A record with 3 IPs and wait until the changes have been replicated
 - route53:
       command: create
       zone: foo.com
@@ -146,6 +166,7 @@ EXAMPLES = '''
       type: A
       ttl: 7200
       value: 1.1.1.1,2.2.2.2,3.3.3.3
+      wait: yes
 
 # Retrieve the details for new.foo.com
 - route53:
@@ -174,6 +195,16 @@ EXAMPLES = '''
       ttl: "7200"
       value: "::1"
 
+# Add a SRV record with multiple fields for a service on port 22222
+# For more information on SRV records see:
+# https://en.wikipedia.org/wiki/SRV_record
+- route53:
+      command: "create"
+      "zone": "foo.com"
+      "record": "_example-service._tcp.foo.com"
+      "type": "SRV"
+      "value": ["0 0 22222 host1.foo.com", "0 0 22222 host2.foo.com"]
+
 # Add a TXT record. Note that TXT and SPF records must be surrounded
 # by quotes when sent to Route 53:
 - route53:
@@ -194,6 +225,36 @@ EXAMPLES = '''
       alias=True
       alias_hosted_zone_id="{{ elb_zone_id }}"
 
+# Retrieve the details for elb.foo.com
+- route53:
+      command: get
+      zone: foo.com
+      record: elb.foo.com
+      type: A
+  register: rec
+
+# Delete an alias record using the results from the get command
+- route53:
+      command: delete
+      zone: foo.com
+      record: "{{ rec.set.record }}"
+      ttl: "{{ rec.set.ttl }}"
+      type: "{{ rec.set.type }}"
+      value: "{{ rec.set.value }}"
+      alias: True
+      alias_hosted_zone_id: "{{ rec.set.alias_hosted_zone_id }}"
+
+# Add an alias record that points to an Amazon ELB and evaluates it health:
+- route53:
+      command=create
+      zone=foo.com
+      record=elb.foo.com
+      type=A
+      value="{{ elb_dns_name }}"
+      alias=True
+      alias_hosted_zone_id="{{ elb_zone_id }}"
+      alias_evaluate_target_health=True
+
 # Add an AAAA record with Hosted Zone ID.  Note that because there are colons in the value
 # that the entire parameter list must be quoted:
 - route53:
@@ -204,7 +265,7 @@ EXAMPLES = '''
       type: "AAAA"
       ttl: "7200"
       value: "::1"
-      
+
 # Add an AAAA record with Hosted Zone ID.  Note that because there are colons in the value
 # that the entire parameter list must be quoted:
 - route53:
@@ -215,7 +276,7 @@ EXAMPLES = '''
       type: "AAAA"
       ttl: "7200"
       value: "::1"
-      
+
 # Use a routing policy to distribute traffic:
 - route53:
       command: "create"
@@ -231,7 +292,12 @@ EXAMPLES = '''
 
 '''
 
+MINIMUM_BOTO_VERSION = '2.28.0'
+WAIT_RETRY_SLEEP = 5  # how many seconds to wait between propagation status polls
+
+
 import time
+import distutils.version
 
 try:
     import boto
@@ -239,10 +305,16 @@ try:
     from boto import route53
     from boto.route53 import Route53Connection
     from boto.route53.record import Record, ResourceRecordSets
+    from boto.route53.status import Status
     HAS_BOTO = True
 except ImportError:
     HAS_BOTO = False
 
+
+class TimeoutError(Exception):
+    pass
+
+
 def get_zone_by_name(conn, module, zone_name, want_private, zone_id, want_vpc_id):
     """Finds a zone by name or zone_id"""
     for zone in conn.get_zones():
@@ -266,41 +338,73 @@ def get_zone_by_name(conn, module, zone_name, want_private, zone_id, want_vpc_id
     return None
 
 
-def commit(changes, retry_interval):
+def commit(changes, retry_interval, wait, wait_timeout):
     """Commit changes, but retry PriorRequestNotComplete errors."""
+    result = None
     retry = 10
     while True:
         try:
             retry -= 1
-            return changes.commit()
-        except boto.route53.exception.DNSServerError, e:
+            result = changes.commit()
+            break
+        except boto.route53.exception.DNSServerError as e:
             code = e.body.split("<Code>")[1]
             code = code.split("</Code>")[0]
             if code != 'PriorRequestNotComplete' or retry < 0:
                 raise e
             time.sleep(float(retry_interval))
 
+    if wait:
+        timeout_time = time.time() + wait_timeout
+        connection = changes.connection
+        change = result['ChangeResourceRecordSetsResponse']['ChangeInfo']
+        status = Status(connection, change)
+        while status.status != 'INSYNC' and time.time() < timeout_time:
+            time.sleep(WAIT_RETRY_SLEEP)
+            status.update()
+        if time.time() >= timeout_time:
+            raise TimeoutError()
+        return result
+
+# Shamelessly copied over from https://git.io/vgmDG
+IGNORE_CODE = 'Throttling'
+MAX_RETRIES=5
+def invoke_with_throttling_retries(function_ref, *argv):
+    retries=0
+    while True:
+        try:
+            retval=function_ref(*argv)
+            return retval
+        except boto.exception.BotoServerError as e:
+            if e.code != IGNORE_CODE or retries==MAX_RETRIES:
+                raise e
+        time.sleep(5 * (2**retries))
+        retries += 1
+
 def main():
     argument_spec = ec2_argument_spec()
     argument_spec.update(dict(
-            command              = dict(choices=['get', 'create', 'delete'], required=True),
-            zone                 = dict(required=True),
-            hosted_zone_id       = dict(required=False, default=None),
-            record               = dict(required=True),
-            ttl                  = dict(required=False, type='int', default=3600),
-            type                 = dict(choices=['A', 'CNAME', 'MX', 'AAAA', 'TXT', 'PTR', 'SRV', 'SPF', 'NS'], required=True),
-            alias                = dict(required=False, type='bool'),
-            alias_hosted_zone_id = dict(required=False),
-            value                = dict(required=False),
-            overwrite            = dict(required=False, type='bool'),
-            retry_interval       = dict(required=False, default=500),
-            private_zone         = dict(required=False, type='bool', default=False),
-            identifier           = dict(required=False, default=None),
-            weight               = dict(required=False, type='int'),
-            region               = dict(required=False),
-            health_check         = dict(required=False),
-            failover             = dict(required=False),
-            vpc_id               = dict(required=False),
+            command                      = dict(choices=['get', 'create', 'delete'], required=True),
+            zone                         = dict(required=True),
+            hosted_zone_id               = dict(required=False, default=None),
+            record                       = dict(required=True),
+            ttl                          = dict(required=False, type='int', default=3600),
+            type                         = dict(choices=['A', 'CNAME', 'MX', 'AAAA', 'TXT', 'PTR', 'SRV', 'SPF', 'NS', 'SOA'], required=True),
+            alias                        = dict(required=False, type='bool'),
+            alias_hosted_zone_id         = dict(required=False),
+            alias_evaluate_target_health = dict(required=False, type='bool', default=False),
+            value                        = dict(required=False),
+            overwrite                    = dict(required=False, type='bool'),
+            retry_interval               = dict(required=False, default=500),
+            private_zone                 = dict(required=False, type='bool', default=False),
+            identifier                   = dict(required=False, default=None),
+            weight                       = dict(required=False, type='int'),
+            region                       = dict(required=False),
+            health_check                 = dict(required=False),
+            failover                     = dict(required=False,choices=['PRIMARY','SECONDARY']),
+            vpc_id                       = dict(required=False),
+            wait                         = dict(required=False, type='bool', default=False),
+            wait_timeout                 = dict(required=False, type='int', default=300),
         )
     )
     module = AnsibleModule(argument_spec=argument_spec)
@@ -308,23 +412,29 @@ def main():
     if not HAS_BOTO:
         module.fail_json(msg='boto required for this module')
 
-    command_in              = module.params.get('command')
-    zone_in                 = module.params.get('zone').lower()
-    hosted_zone_id_in       = module.params.get('hosted_zone_id')
-    ttl_in                  = module.params.get('ttl')
-    record_in               = module.params.get('record').lower()
-    type_in                 = module.params.get('type')
-    value_in                = module.params.get('value')
-    alias_in                = module.params.get('alias')
-    alias_hosted_zone_id_in = module.params.get('alias_hosted_zone_id')
-    retry_interval_in       = module.params.get('retry_interval')
-    private_zone_in         = module.params.get('private_zone')
-    identifier_in           = module.params.get('identifier')
-    weight_in               = module.params.get('weight')
-    region_in               = module.params.get('region')
-    health_check_in         = module.params.get('health_check')
-    failover_in             = module.params.get('failover')
-    vpc_id_in               = module.params.get('vpc_id')
+    if distutils.version.StrictVersion(boto.__version__) < distutils.version.StrictVersion(MINIMUM_BOTO_VERSION):
+        module.fail_json(msg='Found boto in version %s, but >= %s is required' % (boto.__version__, MINIMUM_BOTO_VERSION))
+
+    command_in                      = module.params.get('command')
+    zone_in                         = module.params.get('zone').lower()
+    hosted_zone_id_in               = module.params.get('hosted_zone_id')
+    ttl_in                          = module.params.get('ttl')
+    record_in                       = module.params.get('record').lower()
+    type_in                         = module.params.get('type')
+    value_in                        = module.params.get('value')
+    alias_in                        = module.params.get('alias')
+    alias_hosted_zone_id_in         = module.params.get('alias_hosted_zone_id')
+    alias_evaluate_target_health_in = module.params.get('alias_evaluate_target_health')
+    retry_interval_in               = module.params.get('retry_interval')
+    private_zone_in                 = module.params.get('private_zone')
+    identifier_in                   = module.params.get('identifier')
+    weight_in                       = module.params.get('weight')
+    region_in                       = module.params.get('region')
+    health_check_in                 = module.params.get('health_check')
+    failover_in                     = module.params.get('failover')
+    vpc_id_in                       = module.params.get('vpc_id')
+    wait_in                         = module.params.get('wait')
+    wait_timeout_in                 = module.params.get('wait_timeout')
 
     region, ec2_url, aws_connect_kwargs = get_aws_connection_info(module)
 
@@ -346,20 +456,30 @@ def main():
         if not value_in:
             module.fail_json(msg = "parameter 'value' required for create/delete")
         elif alias_in:
-          if len(value_list) != 1:
-              module.fail_json(msg = "parameter 'value' must contain a single dns name for alias create/delete")
-          elif not alias_hosted_zone_id_in:
-              module.fail_json(msg = "parameter 'alias_hosted_zone_id' required for alias create/delete")
+            if len(value_list) != 1:
+                module.fail_json(msg = "parameter 'value' must contain a single dns name for alias create/delete")
+            elif not alias_hosted_zone_id_in:
+                module.fail_json(msg = "parameter 'alias_hosted_zone_id' required for alias create/delete")
+        elif ( weight_in!=None or region_in!=None or failover_in!=None ) and identifier_in==None:
+            module.fail_json(msg= "If you specify failover, region or weight you must also specify identifier")
+
+    if command_in == 'create':
+        if ( weight_in!=None or region_in!=None or failover_in!=None ) and identifier_in==None:
+          module.fail_json(msg= "If you specify failover, region or weight you must also specify identifier")
+        elif  ( weight_in==None and region_in==None and failover_in==None ) and identifier_in!=None:
+          module.fail_json(msg= "You have specified identifier which makes sense only if you specify one of: weight, region or failover.")
+
+
 
     if vpc_id_in and not private_zone_in:
         module.fail_json(msg="parameter 'private_zone' must be true when specifying parameter"
             " 'vpc_id'")
 
 
-    # connect to the route53 endpoint 
+    # connect to the route53 endpoint
     try:
         conn = Route53Connection(**aws_connect_kwargs)
-    except boto.exception.BotoServerError, e:
+    except boto.exception.BotoServerError as e:
         module.fail_json(msg = e.error_message)
 
     # Find the named zone ID
@@ -371,14 +491,14 @@ def main():
         module.fail_json(msg = errmsg)
 
     record = {}
-    
+
     found_record = False
     wanted_rset = Record(name=record_in, type=type_in, ttl=ttl_in,
         identifier=identifier_in, weight=weight_in, region=region_in,
         health_check=health_check_in, failover=failover_in)
     for v in value_list:
         if alias_in:
-            wanted_rset.set_alias(alias_hosted_zone_id_in, v)
+            wanted_rset.set_alias(alias_hosted_zone_id_in, v, alias_evaluate_target_health_in)
         else:
             wanted_rset.add_value(v)
 
@@ -389,9 +509,12 @@ def main():
         decoded_name = rset.name.replace(r'\052', '*')
         decoded_name = decoded_name.replace(r'\100', '@')
         #Need to save this changes in rset, because of comparing rset.to_xml() == wanted_rset.to_xml() in next block
-	rset.name = decoded_name
+        rset.name = decoded_name
 
-        if rset.type == type_in and decoded_name.lower() == record_in.lower() and str(rset.identifier) == str(identifier_in):
+        if identifier_in is not None:
+            identifier_in = str(identifier_in)
+
+        if rset.type == type_in and decoded_name.lower() == record_in.lower() and rset.identifier == identifier_in:
             found_record = True
             record['zone'] = zone_in
             record['type'] = rset.type
@@ -413,6 +536,7 @@ def main():
               record['value'] = rset.alias_dns_name
               record['values'] = [rset.alias_dns_name]
               record['alias_hosted_zone_id'] = rset.alias_hosted_zone_id
+              record['alias_evaluate_target_health'] = rset.alias_evaluate_target_health
             else:
               record['alias'] = False
               record['value'] = ','.join(sorted(rset.resource_records))
@@ -445,11 +569,16 @@ def main():
         changes.add_change_record(command, wanted_rset)
 
     try:
-        result = commit(changes, retry_interval_in)
-    except boto.route53.exception.DNSServerError, e:
+        result = invoke_with_throttling_retries(commit, changes, retry_interval_in, wait_in, wait_timeout_in)
+    except boto.route53.exception.DNSServerError as e:
         txt = e.body.split("<Message>")[1]
         txt = txt.split("</Message>")[0]
-        module.fail_json(msg = txt)
+        if "but it already exists" in txt:
+                module.exit_json(changed=False)
+        else:
+                module.fail_json(msg = txt)
+    except TimeoutError:
+        module.fail_json(msg='Timeout waiting for changes to replicate')
 
     module.exit_json(changed=True)
 
