@@ -28,9 +28,10 @@ version_added: "1.4"
 short_description: Manage hostname
 requirements: [ hostname ]
 description:
-    - Set system's hostname
+    - Set system's hostname.
     - Currently implemented on Debian, Ubuntu, Fedora, RedHat, openSUSE, Linaro, ScientificLinux, Arch, CentOS, AMI.
-    - Any distribution that uses systemd as their init system
+    - Any distribution that uses systemd as their init system.
+    - Note, this module does *NOT* modify /etc/hosts. You need to modify it yourself using other modules like template or replace.
 options:
     name:
         required: true
@@ -47,6 +48,8 @@ from distutils.version import LooseVersion
 
 # import module snippets
 from ansible.module_utils.basic import *
+from ansible.module_utils.facts import *
+from ansible.module_utils._text import to_bytes, to_native
 
 
 class UnimplementedStrategy(object):
@@ -93,9 +96,12 @@ class Hostname(object):
         return load_platform_subclass(Hostname, args, kwargs)
 
     def __init__(self, module):
-        self.module   = module
-        self.name     = module.params['name']
-        self.strategy = self.strategy_class(module)
+        self.module       = module
+        self.name         = module.params['name']
+        if self.platform == 'Linux' and Facts(module).is_systemd_managed():
+            self.strategy = SystemdStrategy(module)
+        else:
+            self.strategy = self.strategy_class(module)
 
     def get_current_hostname(self):
         return self.strategy.get_current_hostname()
@@ -130,7 +136,7 @@ class GenericStrategy(object):
         if rc != 0:
             self.module.fail_json(msg="Command failed rc=%d, out=%s, err=%s" %
                 (rc, out, err))
-        return out.strip()
+        return to_native(out).strip()
 
     def set_current_hostname(self, name):
         cmd = [self.hostname_cmd, name]
@@ -160,7 +166,8 @@ class DebianStrategy(GenericStrategy):
         if not os.path.isfile(self.HOSTNAME_FILE):
             try:
                 open(self.HOSTNAME_FILE, "a").write("")
-            except IOError, err:
+            except IOError:
+                err = get_exception()
                 self.module.fail_json(msg="failed to write file: %s" %
                     str(err))
         try:
@@ -169,7 +176,8 @@ class DebianStrategy(GenericStrategy):
                 return f.read().strip()
             finally:
                 f.close()
-        except Exception, err:
+        except Exception:
+            err = get_exception()
             self.module.fail_json(msg="failed to read hostname: %s" %
                 str(err))
 
@@ -180,10 +188,50 @@ class DebianStrategy(GenericStrategy):
                 f.write("%s\n" % name)
             finally:
                 f.close()
-        except Exception, err:
+        except Exception:
+            err = get_exception()
             self.module.fail_json(msg="failed to update hostname: %s" %
                 str(err))
 
+# ===========================================
+
+class SLESStrategy(GenericStrategy):
+    """
+    This is a SLES Hostname strategy class - it edits the
+    /etc/HOSTNAME file.
+    """
+    HOSTNAME_FILE = '/etc/HOSTNAME'
+
+    def get_permanent_hostname(self):
+        if not os.path.isfile(self.HOSTNAME_FILE):
+            try:
+                open(self.HOSTNAME_FILE, "a").write("")
+            except IOError:
+                err = get_exception()
+                self.module.fail_json(msg="failed to write file: %s" %
+                    str(err))
+        try:
+            f = open(self.HOSTNAME_FILE)
+            try:
+                return f.read().strip()
+            finally:
+                f.close()
+        except Exception:
+            err = get_exception()
+            self.module.fail_json(msg="failed to read hostname: %s" %
+                str(err))
+
+    def set_permanent_hostname(self, name):
+        try:
+            f = open(self.HOSTNAME_FILE, 'w+')
+            try:
+                f.write("%s\n" % name)
+            finally:
+                f.close()
+        except Exception:
+            err = get_exception()
+            self.module.fail_json(msg="failed to update hostname: %s" %
+                str(err))
 
 # ===========================================
 
@@ -204,7 +252,8 @@ class RedHatStrategy(GenericStrategy):
                         return v.strip()
             finally:
                 f.close()
-        except Exception, err:
+        except Exception:
+            err = get_exception()
             self.module.fail_json(msg="failed to read hostname: %s" %
                 str(err))
 
@@ -229,7 +278,8 @@ class RedHatStrategy(GenericStrategy):
                 f.writelines(lines)
             finally:
                 f.close()
-        except Exception, err:
+        except Exception:
+            err = get_exception()
             self.module.fail_json(msg="failed to update hostname: %s" %
                 str(err))
 
@@ -248,7 +298,7 @@ class SystemdStrategy(GenericStrategy):
         if rc != 0:
             self.module.fail_json(msg="Command failed rc=%d, out=%s, err=%s" %
                 (rc, out, err))
-        return out.strip()
+        return to_native(out).strip()
 
     def set_current_hostname(self, name):
         if len(name) > 64:
@@ -260,12 +310,12 @@ class SystemdStrategy(GenericStrategy):
                 (rc, out, err))
 
     def get_permanent_hostname(self):
-        cmd = 'hostnamectl --static status'
-        rc, out, err = self.module.run_command(cmd, use_unsafe_shell=True)
+        cmd = ['hostnamectl', '--static', 'status']
+        rc, out, err = self.module.run_command(cmd)
         if rc != 0:
             self.module.fail_json(msg="Command failed rc=%d, out=%s, err=%s" %
                 (rc, out, err))
-        return out.strip()
+        return to_native(out).strip()
 
     def set_permanent_hostname(self, name):
         if len(name) > 64:
@@ -300,7 +350,8 @@ class OpenRCStrategy(GenericStrategy):
                     line = line.strip()
                     if line.startswith('hostname='):
                         return line[10:].strip('"')
-            except Exception, err:
+            except Exception:
+                err = get_exception()
                 self.module.fail_json(msg="failed to read hostname: %s" % str(err))
         finally:
             f.close()
@@ -321,7 +372,8 @@ class OpenRCStrategy(GenericStrategy):
 
                 f = open(self.HOSTNAME_FILE, 'w')
                 f.write('\n'.join(lines) + '\n')
-            except Exception, err:
+            except Exception:
+                err = get_exception()
                 self.module.fail_json(msg="failed to update hostname: %s" % str(err))
         finally:
             f.close()
@@ -340,7 +392,8 @@ class OpenBSDStrategy(GenericStrategy):
         if not os.path.isfile(self.HOSTNAME_FILE):
             try:
                 open(self.HOSTNAME_FILE, "a").write("")
-            except IOError, err:
+            except IOError:
+                err = get_exception()
                 self.module.fail_json(msg="failed to write file: %s" %
                     str(err))
         try:
@@ -349,7 +402,8 @@ class OpenBSDStrategy(GenericStrategy):
                 return f.read().strip()
             finally:
                 f.close()
-        except Exception, err:
+        except Exception:
+            err = get_exception()
             self.module.fail_json(msg="failed to read hostname: %s" %
                 str(err))
 
@@ -360,7 +414,8 @@ class OpenBSDStrategy(GenericStrategy):
                 f.write("%s\n" % name)
             finally:
                 f.close()
-        except Exception, err:
+        except Exception:
+            err = get_exception()
             self.module.fail_json(msg="failed to update hostname: %s" %
                 str(err))
 
@@ -388,7 +443,7 @@ class SolarisStrategy(GenericStrategy):
         if rc != 0:
             self.module.fail_json(msg="Command failed rc=%d, out=%s, err=%s" %
                 (rc, out, err))
-        return out.strip()
+        return to_native(out).strip()
 
     def set_permanent_hostname(self, name):
         cmd = [self.hostname_cmd, name]
@@ -399,6 +454,60 @@ class SolarisStrategy(GenericStrategy):
 
 # ===========================================
 
+class FreeBSDStrategy(GenericStrategy):
+    """
+    This is a FreeBSD hostname manipulation strategy class - it edits
+    the /etc/rc.conf.d/hostname file.
+    """
+
+    HOSTNAME_FILE = '/etc/rc.conf.d/hostname'
+
+    def get_permanent_hostname(self):
+
+        if not os.path.isfile(self.HOSTNAME_FILE):
+            try:
+                open(self.HOSTNAME_FILE, "a").write("hostname=temporarystub\n")
+            except IOError:
+                err = get_exception()
+                self.module.fail_json(msg="failed to write file: %s" %
+                    str(err))
+        try:
+            try:
+                f = open(self.HOSTNAME_FILE, 'r')
+                for line in f:
+                    line = line.strip()
+                    if line.startswith('hostname='):
+                        return line[10:].strip('"')
+            except Exception:
+                err = get_exception()
+                self.module.fail_json(msg="failed to read hostname: %s" % str(err))
+        finally:
+            f.close()
+
+        return None
+
+    def set_permanent_hostname(self, name):
+        try:
+            try:
+                f = open(self.HOSTNAME_FILE, 'r')
+                lines = [x.strip() for x in f]
+
+                for i, line in enumerate(lines):
+                    if line.startswith('hostname='):
+                        lines[i] = 'hostname="%s"' % name
+                        break
+                f.close()
+
+                f = open(self.HOSTNAME_FILE, 'w')
+                f.write('\n'.join(lines) + '\n')
+            except Exception:
+                err = get_exception()
+                self.module.fail_json(msg="failed to update hostname: %s" % str(err))
+        finally:
+            f.close()
+
+# ===========================================
+
 class FedoraHostname(Hostname):
     platform = 'Linux'
     distribution = 'Fedora'
@@ -408,8 +517,8 @@ class SLESHostname(Hostname):
     platform = 'Linux'
     distribution = 'Suse linux enterprise server '
     distribution_version = get_distribution_version()
-    if distribution_version and LooseVersion(distribution_version) >= LooseVersion("12"):
-        strategy_class = SystemdStrategy
+    if distribution_version and LooseVersion("10") <= LooseVersion(distribution_version) <= LooseVersion("12"):
+        strategy_class = SLESStrategy
     else:
         strategy_class = UnimplementedStrategy
 
@@ -431,65 +540,42 @@ class RedHat5Hostname(Hostname):
 class RedHatServerHostname(Hostname):
     platform = 'Linux'
     distribution = 'Red hat enterprise linux server'
-    distribution_version = get_distribution_version()
-    if distribution_version and LooseVersion(distribution_version) >= LooseVersion("7"):
-        strategy_class = SystemdStrategy
-    else:
-        strategy_class = RedHatStrategy
+    strategy_class = RedHatStrategy
 
 class RedHatWorkstationHostname(Hostname):
     platform = 'Linux'
     distribution = 'Red hat enterprise linux workstation'
-    distribution_version = get_distribution_version()
-    if distribution_version and LooseVersion(distribution_version) >= LooseVersion("7"):
-        strategy_class = SystemdStrategy
-    else:
-        strategy_class = RedHatStrategy
+    strategy_class = RedHatStrategy
 
 class CentOSHostname(Hostname):
     platform = 'Linux'
     distribution = 'Centos'
-    distribution_version = get_distribution_version()
-    if distribution_version and LooseVersion(distribution_version) >= LooseVersion("7"):
-        strategy_class = SystemdStrategy
-    else:
-        strategy_class = RedHatStrategy
+    strategy_class = RedHatStrategy
 
 class CentOSLinuxHostname(Hostname):
     platform = 'Linux'
     distribution = 'Centos linux'
-    distribution_version = get_distribution_version()
-    if distribution_version and LooseVersion(distribution_version) >= LooseVersion("7"):
-        strategy_class = SystemdStrategy
-    else:
-        strategy_class = RedHatStrategy
+    strategy_class = RedHatStrategy
 
 class ScientificHostname(Hostname):
     platform = 'Linux'
     distribution = 'Scientific'
-    distribution_version = get_distribution_version()
-    if distribution_version and LooseVersion(distribution_version) >= LooseVersion("7"):
-        strategy_class = SystemdStrategy
-    else:
-        strategy_class = RedHatStrategy
+    strategy_class = RedHatStrategy
 
 class ScientificLinuxHostname(Hostname):
     platform = 'Linux'
     distribution = 'Scientific linux'
-    distribution_version = get_distribution_version()
-    if distribution_version and LooseVersion(distribution_version) >= LooseVersion("7"):
-        strategy_class = SystemdStrategy
-    else:
-        strategy_class = RedHatStrategy
+    strategy_class = RedHatStrategy
+
+class ScientificLinuxCERNHostname(Hostname):
+    platform = 'Linux'
+    distribution = 'Scientific linux cern slc'
+    strategy_class = RedHatStrategy
 
 class OracleLinuxHostname(Hostname):
     platform = 'Linux'
     distribution = 'Oracle linux server'
-    distribution_version = get_distribution_version()
-    if distribution_version and LooseVersion(distribution_version) >= LooseVersion("7"):
-        strategy_class = SystemdStrategy
-    else:
-        strategy_class = RedHatStrategy
+    strategy_class = RedHatStrategy
 
 class AmazonLinuxHostname(Hostname):
     platform = 'Linux'
@@ -541,12 +627,18 @@ class SolarisHostname(Hostname):
     distribution = None
     strategy_class = SolarisStrategy
 
+class FreeBSDHostname(Hostname):
+    platform = 'FreeBSD'
+    distribution = None
+    strategy_class = FreeBSDStrategy
+
+
 # ===========================================
 
 def main():
     module = AnsibleModule(
         argument_spec = dict(
-            name=dict(required=True, type='str')
+            name=dict(required=True)
         )
     )
 
@@ -570,4 +662,5 @@ def main():
                                         ansible_fqdn=socket.getfqdn(),
                                         ansible_domain='.'.join(socket.getfqdn().split('.')[1:])))
 
-main()
+if __name__ == '__main__':
+    main()
