@@ -42,6 +42,12 @@ options:
       - VM guest NIC secondary IP address for the static NAT.
     required: false
     default: false
+  network:
+    description:
+      - Network the IP address is related to.
+    required: false
+    default: null
+    version_added: "2.2"
   state:
     description:
       - State of the static NAT.
@@ -140,13 +146,6 @@ domain:
   sample: example domain
 '''
 
-
-try:
-    from cs import CloudStack, CloudStackException, read_config
-    has_lib_cs = True
-except ImportError:
-    has_lib_cs = False
-
 # import cloudstack common
 from ansible.module_utils.cloudstack import *
 
@@ -161,35 +160,6 @@ class AnsibleCloudStackStaticNat(AnsibleCloudStack):
             'ipaddress':                    'ip_address',
             'vmipaddress':                  'vm_guest_ip',
         }
-        self.vm_default_nic = None
-
-
-# TODO: move it to cloudstack utils, also used in cs_portforward
-    def get_vm_guest_ip(self):
-        vm_guest_ip = self.module.params.get('vm_guest_ip')
-        default_nic = self.get_vm_default_nic()
-
-        if not vm_guest_ip:
-            return default_nic['ipaddress']
-
-        for secondary_ip in default_nic['secondaryip']:
-            if vm_guest_ip == secondary_ip['ipaddress']:
-                return vm_guest_ip
-        self.module.fail_json(msg="Secondary IP '%s' not assigned to VM" % vm_guest_ip)
-
-
-# TODO: move it to cloudstack utils, also used in cs_portforward
-    def get_vm_default_nic(self):
-        if self.vm_default_nic:
-            return self.vm_default_nic
-
-        nics = self.cs.listNics(virtualmachineid=self.get_vm(key='id'))
-        if nics:
-            for n in nics['nic']:
-                if n['isdefault']:
-                    self.vm_default_nic = n
-                    return self.vm_default_nic
-        self.module.fail_json(msg="No default IP address of VM '%s' found" % self.module.params.get('vm'))
 
 
     def create_static_nat(self, ip_address):
@@ -198,6 +168,7 @@ class AnsibleCloudStackStaticNat(AnsibleCloudStack):
         args['virtualmachineid'] = self.get_vm(key='id')
         args['ipaddressid'] = ip_address['id']
         args['vmguestip'] = self.get_vm_guest_ip()
+        args['networkid'] = self.get_network(key='id')
         if not self.module.check_mode:
             res = self.cs.enableStaticNat(**args)
             if 'errortext' in res:
@@ -217,13 +188,13 @@ class AnsibleCloudStackStaticNat(AnsibleCloudStack):
 
         # make an alias, so we can use _has_changed()
         ip_address['vmguestip'] = ip_address['vmipaddress']
-        if self._has_changed(args, ip_address, ['vmguestip', 'virtualmachineid']):
+        if self.has_changed(args, ip_address, ['vmguestip', 'virtualmachineid']):
             self.result['changed'] = True
             if not self.module.check_mode:
                 res = self.cs.disableStaticNat(ipaddressid=ip_address['id'])
                 if 'errortext' in res:
                     self.module.fail_json(msg="Failed: '%s'" % res['errortext'])
-                res = self._poll_job(res, 'staticnat')
+                self.poll_job(res, 'staticnat')
                 res = self.cs.enableStaticNat(**args)
                 if 'errortext' in res:
                     self.module.fail_json(msg="Failed: '%s'" % res['errortext'])
@@ -253,17 +224,17 @@ class AnsibleCloudStackStaticNat(AnsibleCloudStack):
                     self.module.fail_json(msg="Failed: '%s'" % res['errortext'])
                 poll_async = self.module.params.get('poll_async')
                 if poll_async:
-                    res = self._poll_job(res, 'staticnat')
+                    self.poll_job(res, 'staticnat')
         return ip_address
 
 
-
 def main():
     argument_spec = cs_argument_spec()
     argument_spec.update(dict(
         ip_address = dict(required=True),
         vm = dict(default=None),
         vm_guest_ip = dict(default=None),
+        network = dict(default=None),
         state = dict(choices=['present', 'absent'], default='present'),
         zone = dict(default=None),
         domain = dict(default=None),
@@ -278,9 +249,6 @@ def main():
         supports_check_mode=True
     )
 
-    if not has_lib_cs:
-        module.fail_json(msg="python library cs required: pip install cs")
-
     try:
         acs_static_nat = AnsibleCloudStackStaticNat(module)
 
