@@ -20,6 +20,7 @@ module: proxmox
 short_description: management of instances in Proxmox VE cluster
 description:
   - allows you to create/delete/stop instances in Proxmox VE cluster
+  - Starting in Ansible 2.1, it automatically detects containerization type (lxc for PVE 4, openvz for older)
 version_added: "2.0"
 options:
   api_host:
@@ -97,7 +98,14 @@ options:
       - specifies network interfaces for the container
     default: null
     required: false
-    type: string
+    type:  A hash/dictionary defining interfaces
+  mounts:
+    description:
+      - specifies additional mounts (separate disks) for the container
+    default: null
+    required: false
+    type:  A hash/dictionary defining mount points
+    version_added: "2.2"
   ip_address:
     description:
       - specifies the address the container will be assigned
@@ -170,6 +178,12 @@ EXAMPLES = '''
 # Create new container with minimal options use environment PROXMOX_PASSWORD variable(you should export it before)
 - proxmox: vmid=100 node='uk-mc02' api_user='root@pam' api_host='node1' password='123456' hostname='example.org' ostemplate='local:vztmpl/ubuntu-14.04-x86_64.tar.gz'
 
+# Create new container with minimal options defining network interface with dhcp
+- proxmox: vmid=100 node='uk-mc02' api_user='root@pam' api_password='1q2w3e' api_host='node1' password='123456' hostname='example.org' ostemplate='local:vztmpl/ubuntu-14.04-x86_64.tar.gz' netif='{"net0":"name=eth0,ip=dhcp,ip6=dhcp,bridge=vmbr0"}'
+
+# Create new container with minimal options defining a mount
+- proxmox: vmid=100 node='uk-mc02' api_user='root@pam' api_password='1q2w3e' api_host='node1' password='123456' hostname='example.org' ostemplate='local:vztmpl/ubuntu-14.04-x86_64.tar.gz' mounts='{"mp0":"local:8,mp=/mnt/test/"}'
+
 # Start container
 - proxmox: vmid=100 api_user='root@pam' api_password='1q2w3e' api_host='node1' state=started
 
@@ -195,19 +209,33 @@ try:
 except ImportError:
   HAS_PROXMOXER = False
 
+VZ_TYPE=None
+
 def get_instance(proxmox, vmid):
   return [ vm for vm in proxmox.cluster.resources.get(type='vm') if vm['vmid'] == int(vmid) ]
 
-def content_check(proxmox, node, ostemplate, storage):
-  return [ True for cnt in proxmox.nodes(node).storage(storage).content.get() if cnt['volid'] == ostemplate ]
+def content_check(proxmox, node, ostemplate, template_store):
+  return [ True for cnt in proxmox.nodes(node).storage(template_store).content.get() if cnt['volid'] == ostemplate ]
 
 def node_check(proxmox, node):
   return [ True for nd in proxmox.nodes.get() if nd['node'] == node ]
 
 def create_instance(module, proxmox, vmid, node, disk, storage, cpus, memory, swap, timeout, **kwargs):
   proxmox_node = proxmox.nodes(node)
-  taskid = proxmox_node.openvz.create(vmid=vmid, storage=storage, memory=memory, swap=swap,
-                             cpus=cpus, disk=disk, **kwargs)
+  kwargs = dict((k,v) for k, v in kwargs.items() if v is not None)
+  if VZ_TYPE =='lxc':
+      kwargs['cpulimit']=cpus
+      kwargs['rootfs']=disk
+      if 'netif' in kwargs:
+        kwargs.update(kwargs['netif'])
+        del kwargs['netif']
+      if 'mounts' in kwargs:
+        kwargs.update(kwargs['mounts'])
+        del kwargs['mounts']
+  else:
+      kwargs['cpus']=cpus
+      kwargs['disk']=disk
+  taskid = getattr(proxmox_node, VZ_TYPE).create(vmid=vmid, storage=storage, memory=memory, swap=swap, **kwargs)
 
   while timeout:
     if ( proxmox_node.tasks(taskid).status.get()['status'] == 'stopped'
@@ -222,7 +250,7 @@ def create_instance(module, proxmox, vmid, node, disk, storage, cpus, memory, sw
   return False
 
 def start_instance(module, proxmox, vm, vmid, timeout):
-  taskid = proxmox.nodes(vm[0]['node']).openvz(vmid).status.start.post()
+  taskid = getattr(proxmox.nodes(vm[0]['node']), VZ_TYPE)(vmid).status.start.post()
   while timeout:
     if ( proxmox.nodes(vm[0]['node']).tasks(taskid).status.get()['status'] == 'stopped'
         and proxmox.nodes(vm[0]['node']).tasks(taskid).status.get()['exitstatus'] == 'OK' ):
@@ -237,9 +265,9 @@ def start_instance(module, proxmox, vm, vmid, timeout):
 
 def stop_instance(module, proxmox, vm, vmid, timeout, force):
   if force:
-    taskid = proxmox.nodes(vm[0]['node']).openvz(vmid).status.shutdown.post(forceStop=1)
+    taskid = getattr(proxmox.nodes(vm[0]['node']), VZ_TYPE)(vmid).status.shutdown.post(forceStop=1)
   else:
-    taskid = proxmox.nodes(vm[0]['node']).openvz(vmid).status.shutdown.post()
+    taskid = getattr(proxmox.nodes(vm[0]['node']), VZ_TYPE)(vmid).status.shutdown.post()
   while timeout:
     if ( proxmox.nodes(vm[0]['node']).tasks(taskid).status.get()['status'] == 'stopped'
         and proxmox.nodes(vm[0]['node']).tasks(taskid).status.get()['exitstatus'] == 'OK' ):
@@ -253,7 +281,7 @@ def stop_instance(module, proxmox, vm, vmid, timeout, force):
   return False
 
 def umount_instance(module, proxmox, vm, vmid, timeout):
-  taskid = proxmox.nodes(vm[0]['node']).openvz(vmid).status.umount.post()
+  taskid = getattr(proxmox.nodes(vm[0]['node']), VZ_TYPE)(vmid).status.umount.post()
   while timeout:
     if ( proxmox.nodes(vm[0]['node']).tasks(taskid).status.get()['status'] == 'stopped'
         and proxmox.nodes(vm[0]['node']).tasks(taskid).status.get()['exitstatus'] == 'OK' ):
@@ -273,24 +301,25 @@ def main():
       api_user = dict(required=True),
       api_password = dict(no_log=True),
       vmid = dict(required=True),
-      validate_certs = dict(type='bool', choices=BOOLEANS, default='no'),
+      validate_certs = dict(type='bool', default='no'),
       node = dict(),
       password = dict(no_log=True),
       hostname = dict(),
       ostemplate = dict(),
-      disk = dict(type='int', default=3),
+      disk = dict(type='str', default='3'),
       cpus = dict(type='int', default=1),
       memory = dict(type='int', default=512),
       swap = dict(type='int', default=0),
-      netif = dict(),
+      netif = dict(type='dict'),
+      mounts = dict(type='dict'),
       ip_address = dict(),
-      onboot = dict(type='bool', choices=BOOLEANS, default='no'),
+      onboot = dict(type='bool', default='no'),
       storage = dict(default='local'),
       cpuunits = dict(type='int', default=1000),
       nameserver = dict(),
       searchdomain = dict(),
       timeout = dict(type='int', default=30),
-      force = dict(type='bool', choices=BOOLEANS, default='no'),
+      force = dict(type='bool', default='no'),
       state = dict(default='present', choices=['present', 'absent', 'stopped', 'started', 'restarted']),
     )
   )
@@ -310,18 +339,23 @@ def main():
   memory = module.params['memory']
   swap = module.params['swap']
   storage = module.params['storage']
+  if module.params['ostemplate'] is not None:
+    template_store = module.params['ostemplate'].split(":")[0]
   timeout = module.params['timeout']
 
   # If password not set get it from PROXMOX_PASSWORD env
   if not api_password:
     try:
       api_password = os.environ['PROXMOX_PASSWORD']
-    except KeyError, e:
+    except KeyError as e:
       module.fail_json(msg='You should set api_password param or use PROXMOX_PASSWORD environment variable')
 
   try:
     proxmox = ProxmoxAPI(api_host, user=api_user, password=api_password, verify_ssl=validate_certs)
-  except Exception, e:
+    global VZ_TYPE
+    VZ_TYPE = 'openvz' if float(proxmox.version.get()['version']) < 4.0 else 'lxc'
+
+  except Exception as e:
     module.fail_json(msg='authorization on proxmox cluster failed with exception: %s' % e)
 
   if state == 'present':
@@ -332,15 +366,16 @@ def main():
         module.fail_json(msg='node, hostname, password and ostemplate are mandatory for creating vm')
       elif not node_check(proxmox, node):
         module.fail_json(msg="node '%s' not exists in cluster" % node)
-      elif not content_check(proxmox, node, module.params['ostemplate'], storage):
+      elif not content_check(proxmox, node, module.params['ostemplate'], template_store):
         module.fail_json(msg="ostemplate '%s' not exists on node %s and storage %s"
-                         % (module.params['ostemplate'], node, storage))
+                         % (module.params['ostemplate'], node, template_store))
 
       create_instance(module, proxmox, vmid, node, disk, storage, cpus, memory, swap, timeout,
                       password = module.params['password'],
                       hostname = module.params['hostname'],
                       ostemplate = module.params['ostemplate'],
                       netif = module.params['netif'],
+                      mounts = module.params['mounts'],
                       ip_address = module.params['ip_address'],
                       onboot = int(module.params['onboot']),
                       cpuunits = module.params['cpuunits'],
@@ -349,20 +384,20 @@ def main():
                       force = int(module.params['force']))
 
       module.exit_json(changed=True, msg="deployed VM %s from template %s"  % (vmid, module.params['ostemplate']))
-    except Exception, e:
-      module.fail_json(msg="creation of VM %s failed with exception: %s" % ( vmid, e ))
+    except Exception as e:
+      module.fail_json(msg="creation of %s VM %s failed with exception: %s" % ( VZ_TYPE, vmid, e ))
 
   elif state == 'started':
     try:
       vm = get_instance(proxmox, vmid)
       if not vm:
         module.fail_json(msg='VM with vmid = %s not exists in cluster' % vmid)
-      if proxmox.nodes(vm[0]['node']).openvz(vmid).status.current.get()['status'] == 'running':
+      if getattr(proxmox.nodes(vm[0]['node']), VZ_TYPE)(vmid).status.current.get()['status'] == 'running':
         module.exit_json(changed=False, msg="VM %s is already running" % vmid)
 
       if start_instance(module, proxmox, vm, vmid, timeout):
         module.exit_json(changed=True, msg="VM %s started" % vmid)
-    except Exception, e:
+    except Exception as e:
       module.fail_json(msg="starting of VM %s failed with exception: %s" % ( vmid, e ))
 
   elif state == 'stopped':
@@ -371,7 +406,7 @@ def main():
       if not vm:
         module.fail_json(msg='VM with vmid = %s not exists in cluster' % vmid)
 
-      if proxmox.nodes(vm[0]['node']).openvz(vmid).status.current.get()['status'] == 'mounted':
+      if getattr(proxmox.nodes(vm[0]['node']), VZ_TYPE)(vmid).status.current.get()['status'] == 'mounted':
         if module.params['force']:
           if umount_instance(module, proxmox, vm, vmid, timeout):
             module.exit_json(changed=True, msg="VM %s is shutting down" % vmid)
@@ -379,12 +414,12 @@ def main():
           module.exit_json(changed=False, msg=("VM %s is already shutdown, but mounted. "
                                                "You can use force option to umount it.") % vmid)
 
-      if proxmox.nodes(vm[0]['node']).openvz(vmid).status.current.get()['status'] == 'stopped':
+      if getattr(proxmox.nodes(vm[0]['node']), VZ_TYPE)(vmid).status.current.get()['status'] == 'stopped':
         module.exit_json(changed=False, msg="VM %s is already shutdown" % vmid)
 
       if stop_instance(module, proxmox, vm, vmid, timeout, force = module.params['force']):
         module.exit_json(changed=True, msg="VM %s is shutting down" % vmid)
-    except Exception, e:
+    except Exception as e:
       module.fail_json(msg="stopping of VM %s failed with exception: %s" % ( vmid, e ))
 
   elif state == 'restarted':
@@ -392,14 +427,14 @@ def main():
       vm = get_instance(proxmox, vmid)
       if not vm:
         module.fail_json(msg='VM with vmid = %s not exists in cluster' % vmid)
-      if ( proxmox.nodes(vm[0]['node']).openvz(vmid).status.current.get()['status'] == 'stopped'
-          or proxmox.nodes(vm[0]['node']).openvz(vmid).status.current.get()['status'] == 'mounted' ):
+      if ( getattr(proxmox.nodes(vm[0]['node']), VZ_TYPE)(vmid).status.current.get()['status'] == 'stopped'
+          or getattr(proxmox.nodes(vm[0]['node']), VZ_TYPE)(vmid).status.current.get()['status'] == 'mounted' ):
         module.exit_json(changed=False, msg="VM %s is not running" % vmid)
 
       if ( stop_instance(module, proxmox, vm, vmid, timeout, force = module.params['force']) and
           start_instance(module, proxmox, vm, vmid, timeout) ):
         module.exit_json(changed=True, msg="VM %s is restarted" % vmid)
-    except Exception, e:
+    except Exception as e:
       module.fail_json(msg="restarting of VM %s failed with exception: %s" % ( vmid, e ))
 
   elif state == 'absent':
@@ -408,13 +443,13 @@ def main():
       if not vm:
         module.exit_json(changed=False, msg="VM %s does not exist" % vmid)
 
-      if proxmox.nodes(vm[0]['node']).openvz(vmid).status.current.get()['status'] == 'running':
+      if getattr(proxmox.nodes(vm[0]['node']), VZ_TYPE)(vmid).status.current.get()['status'] == 'running':
         module.exit_json(changed=False, msg="VM %s is running. Stop it before deletion." % vmid)
 
-      if proxmox.nodes(vm[0]['node']).openvz(vmid).status.current.get()['status'] == 'mounted':
+      if getattr(proxmox.nodes(vm[0]['node']), VZ_TYPE)(vmid).status.current.get()['status'] == 'mounted':
         module.exit_json(changed=False, msg="VM %s is mounted. Stop it with force option before deletion." % vmid)
 
-      taskid = proxmox.nodes(vm[0]['node']).openvz.delete(vmid)
+      taskid = getattr(proxmox.nodes(vm[0]['node']), VZ_TYPE).delete(vmid)
       while timeout:
         if ( proxmox.nodes(vm[0]['node']).tasks(taskid).status.get()['status'] == 'stopped'
             and proxmox.nodes(vm[0]['node']).tasks(taskid).status.get()['exitstatus'] == 'OK' ):
@@ -425,7 +460,7 @@ def main():
                            % proxmox_node.tasks(taskid).log.get()[:1])
 
         time.sleep(1)
-    except Exception, e:
+    except Exception as e:
       module.fail_json(msg="deletion of VM %s failed with exception: %s" % ( vmid, e ))
 
 # import module snippets
