@@ -36,7 +36,6 @@ options:
   state:
     description:
       - State of the package on the system
-    required: false
     choices:
       - present
       - absent
@@ -44,7 +43,6 @@ options:
   force:
     description:
       - Forces install of the package (even if it already exists). Using Force will cause ansible to always report that a change was made
-    required: false
     choices:
       - yes
       - no
@@ -52,7 +50,6 @@ options:
   upgrade:
     description:
       - If package is already installed it, try to upgrade to the latest version or to the specified version
-    required: false
     choices:
       - yes
       - no
@@ -61,29 +58,32 @@ options:
     description:
       - Specific version of the package to be installed
       - Ignored when state == 'absent'
-    required: false
-    default: null
   source:
     description:
       - Specify source rather than using default chocolatey repository
-    require: false
-    default: null
   install_args:
     description:
       - Arguments to pass to the native installer
-    require: false
-    default: null
     version_added: '2.1'
   params:
     description:
       - Parameters to pass to the package
-    require: false
-    default: null
     version_added: '2.1'
+  allow_empty_checksums:
+    description:
+      - Allow empty Checksums to be used 
+    require: false
+    default: false
+    version_added: '2.2'
+  ignore_checksums:
+    description:
+      - Ignore Checksums 
+    require: false
+    default: false
+    version_added: '2.2'      
   ignore_dependencies:
     description:
       - Ignore dependencies, only install/upgrade the package itself
-    require: false
     default: false
     version_added: '2.1'
 author: "Trond Hindenes (@trondhindenes), Peter Mounce (@petemounce), Pepe Barbe (@elventear), Adam Keech (@smadam813)"
