@@ -22,6 +22,7 @@ __metaclass__ = type
 import os
 
 from ansible.errors import AnsibleError
+from ansible.playbook.task_include import TaskInclude
 from ansible.template import Templar
 
 try:
@@ -60,8 +61,11 @@ class IncludedFile:
 
         for res in results:
 
-            if res._task.action == 'include':
-                if res._task.loop:
+            original_host = res._host
+            original_task = res._task
+
+            if original_task.action == 'include':
+                if original_task.loop:
                     if 'results' not in res._result:
                         continue
                     include_results = res._result['results']
@@ -73,29 +77,30 @@ class IncludedFile:
                     if 'skipped' in include_result and include_result['skipped'] or 'failed' in include_result:
                         continue
 
-                    original_host = get_original_host(res._host)
-                    original_task = iterator.get_original_task(original_host, res._task)
-
                     task_vars = variable_manager.get_vars(loader=loader, play=iterator._play, host=original_host, task=original_task)
                     templar = Templar(loader=loader, variables=task_vars)
 
                     include_variables = include_result.get('include_variables', dict())
                     loop_var = 'item'
-                    if res._task.loop_control:
-                        loop_var = res._task.loop_control.loop_var or 'item'
+                    if original_task.loop_control:
+                        loop_var = original_task.loop_control.loop_var or 'item'
                     if loop_var in include_result:
                         task_vars[loop_var] = include_variables[loop_var] = include_result[loop_var]
 
+                    include_file = None
                     if original_task:
                         if original_task.static:
                             continue
 
-                        if original_task._task_include:
+                        if original_task._parent:
                             # handle relative includes by walking up the list of parent include
                             # tasks and checking the relative result to see if it exists
-                            parent_include = original_task._task_include
+                            parent_include = original_task._parent
                             cumulative_path = None
                             while parent_include is not None:
+                                if not isinstance(parent_include, TaskInclude):
+                                    parent_include = parent_include._parent
+                                    continue
                                 parent_include_dir = templar.template(os.path.dirname(parent_include.args.get('_raw_params')))
                                 if cumulative_path is None:
                                     cumulative_path = parent_include_dir
@@ -111,14 +116,14 @@ class IncludedFile:
                                 if os.path.exists(include_file):
                                     break
                                 else:
-                                    parent_include = parent_include._task_include
-                        elif original_task._role:
+                                    parent_include = parent_include._parent
+
+                    if include_file is None:
+                        if original_task._role:
                             include_target = templar.template(include_result['include'])
                             include_file = loader.path_dwim_relative(original_task._role._role_path, 'tasks', include_target)
                         else:
                             include_file = loader.path_dwim(include_result['include'])
-                    else:
-                        include_file = loader.path_dwim(include_result['include'])
 
                     include_file = templar.template(include_file)
                     inc_file = IncludedFile(include_file, include_variables, original_task)
