@@ -20,19 +20,17 @@ from __future__ import (absolute_import, division, print_function)
 __metaclass__ = type
 
 import collections
-import sys
 
-from jinja2 import Undefined as j2undefined
+from jinja2.exceptions import UndefinedError
 
 from ansible import constants as C
-from ansible.inventory.host import Host
 from ansible.template import Templar
 
 STATIC_VARS = [
-  'inventory_hostname', 'inventory_hostname_short',
-  'inventory_file', 'inventory_dir', 'playbook_dir',
-  'ansible_play_hosts', 'play_hosts', 'groups', 'ungrouped', 'group_names',
-  'ansible_version', 'omit', 'role_names'
+    'inventory_hostname', 'inventory_hostname_short',
+    'inventory_file', 'inventory_dir', 'playbook_dir',
+    'ansible_play_hosts', 'play_hosts', 'groups',
+    'ungrouped', 'group_names', 'ansible_version', 'omit', 'role_names'
 ]
 
 try:
@@ -62,19 +60,25 @@ class HostVars(collections.Mapping):
         self._inventory = inventory
 
     def _find_host(self, host_name):
-        if host_name in C.LOCALHOST:
+        if host_name in C.LOCALHOST and self._inventory.localhost:
             host = self._inventory.localhost
         else:
             host = self._inventory.get_host(host_name)
         return host
 
-    def __getitem__(self, host_name):
+    def raw_get(self, host_name):
+        '''
+        Similar to __getitem__, however the returned data is not run through
+        the templating engine to expand variables in the hostvars.
+        '''
         host = self._find_host(host_name)
         if host is None:
-            raise j2undefined
+            raise UndefinedError("%s not found in hostvars" % host_name)
 
-        data = self._variable_manager.get_vars(loader=self._loader, host=host, include_hostvars=False)
+        return self._variable_manager.get_vars(loader=self._loader, host=host, include_hostvars=False)
 
+    def __getitem__(self, host_name):
+        data = self.raw_get(host_name)
         sha1_hash = sha1(str(data).encode('utf-8')).hexdigest()
         if sha1_hash in self._cached_result:
             result = self._cached_result[sha1_hash]
@@ -97,15 +101,15 @@ class HostVars(collections.Mapping):
         return self._find_host(host_name) is not None
 
     def __iter__(self):
-        for host in self._inventory.get_hosts(ignore_limits_and_restrictions=True):
-            yield host
+        for host in self._inventory.get_hosts(ignore_limits=True, ignore_restrictions=True):
+            yield host.name
 
     def __len__(self):
-        return len(self._inventory.get_hosts(ignore_limits_and_restrictions=True))
+        return len(self._inventory.get_hosts(ignore_limits=True, ignore_restrictions=True))
 
     def __repr__(self):
         out = {}
-        for host in self._inventory.get_hosts(ignore_limits_and_restrictions=True):
+        for host in self._inventory.get_hosts(ignore_limits=True, ignore_restrictions=True):
             name = host.name
             out[name] = self.get(name)
         return repr(out)
